@@ -4,12 +4,12 @@ from datetime import datetime
 
 import numpy as np
 import pandas as pd
+from helper import iter_dates  # type: ignore
 from sqlglot.expressions import to_column
 
-from examples.sushi.helper import iter_dates
 from sqlmesh import ExecutionContext, model
 from sqlmesh.core.model import IncrementalByTimeRangeKind
-from sqlmesh.utils.date import to_ds
+from sqlmesh.utils.date import to_date
 
 ITEMS = [
     "Ahi",
@@ -49,27 +49,32 @@ ITEMS = [
 
 @model(
     "sushi.items",
-    kind=IncrementalByTimeRangeKind(time_column="ds"),
-    start="3 months ago",
+    kind=IncrementalByTimeRangeKind(time_column="event_date", batch_size=30),
+    start="1 week ago",
     cron="@daily",
-    batch_size=30,
     columns={
         "id": "int",
         "name": "text",
         "price": "double",
-        "ds": "text",
+        "event_date": "date",
     },
     audits=[
-        ("accepted_values", {"column": to_column("name"), "values": ITEMS}),
+        ("accepted_values", {"column": to_column("name"), "is_in": ITEMS}),
         ("not_null", {"columns": [to_column("name"), to_column("price")]}),
         ("assert_items_price_exceeds_threshold", {"price": 0}),
     ],
+    session_properties={
+        "string_prop": "some_value",
+        "int_prop": 1,
+        "float_prop": 1.0,
+        "bool_prop": True,
+    },
 )
 def execute(
     context: ExecutionContext,
     start: datetime,
     end: datetime,
-    latest: datetime,
+    execution_time: datetime,
     **kwargs: t.Any,
 ) -> pd.DataFrame:
     dfs = []
@@ -80,7 +85,7 @@ def execute(
                 {
                     "name": random.sample(ITEMS, num_items),
                     "price": np.random.uniform(3.0, 10.0, size=num_items).round(2),
-                    "ds": to_ds(dt),
+                    "event_date": to_date(dt),
                 }
             )
             .reset_index()
