@@ -6,7 +6,7 @@
 
 SELECT
   waiter_id::INT AS waiter_id,
-  revenue::DOUBLE AS {{ var("top_waiters:revenue") }}
+  revenue::DOUBLE AS revenue
 FROM {{ ref('waiter_revenue_by_day') }}
 WHERE
   ds = (
