@@ -20,7 +20,7 @@ LEFT JOIN {{ ref('items') }} AS i
   WHERE
     o.ds > (select max(ds) from {{ this }})
 {% endif %}
-{% if sqlmesh is defined %}
+{% if sqlmesh_incremental is defined %}
   WHERE
       o.ds BETWEEN '{{ start_ds }}' AND '{{ end_ds }}'
 {% endif %}
