@@ -1,24 +1,30 @@
 """
 .. include:: ../README.md
 """
+
 from __future__ import annotations
 
+import glob
 import logging
 import os
 import sys
+import typing as t
+from datetime import datetime
 from enum import Enum
+from pathlib import Path
 
 from sqlmesh.core.dialect import extend_sqlglot
 
 extend_sqlglot()
 
+from sqlmesh.core import constants as c
 from sqlmesh.core.config import Config
 from sqlmesh.core.context import Context, ExecutionContext
 from sqlmesh.core.engine_adapter import EngineAdapter
-from sqlmesh.core.hooks import hook
 from sqlmesh.core.macros import macro
 from sqlmesh.core.model import Model, model
 from sqlmesh.core.snapshot import Snapshot
+from sqlmesh.utils import debug_mode_enabled, enable_debug_mode
 
 try:
     from sqlmesh._version import __version__, __version_tuple__  # type: ignore
@@ -33,6 +39,7 @@ class RuntimeEnv(str, Enum):
     DATABRICKS = "databricks"
     GOOGLE_COLAB = "google_colab"  # Not currently officially supported
     JUPYTER = "jupyter"
+    DEBUGGER = "debugger"
 
     @classmethod
     def get(cls) -> RuntimeEnv:
@@ -52,6 +59,8 @@ class RuntimeEnv(str, Enum):
         except NameError:
             pass
 
+        if debug_mode_enabled():
+            return RuntimeEnv.DEBUGGER
         return RuntimeEnv.TERMINAL
 
     @property
@@ -75,10 +84,7 @@ class RuntimeEnv(str, Enum):
         return not self.is_terminal
 
 
-runtime_env = RuntimeEnv.get()
-
-
-if runtime_env.is_notebook:
+if RuntimeEnv.get().is_notebook:
     try:
         from sqlmesh.magics import register_magics
 
@@ -87,6 +93,10 @@ if runtime_env.is_notebook:
         pass
 
 
+LOG_FORMAT = "%(asctime)s - %(threadName)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
+LOG_FILENAME_PREFIX = "sqlmesh_"
+
+
 # SO: https://stackoverflow.com/questions/384076/how-can-i-color-python-logging-output
 class CustomFormatter(logging.Formatter):
     """Custom logging formatter."""
@@ -96,14 +106,13 @@ class CustomFormatter(logging.Formatter):
     red = "\x1b[31;20m"
     bold_red = "\x1b[31;1m"
     reset = "\x1b[0m"
-    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
 
     FORMATS = {
-        logging.DEBUG: grey + log_format + reset,
-        logging.INFO: grey + log_format + reset,
-        logging.WARNING: yellow + log_format + reset,
-        logging.ERROR: red + log_format + reset,
-        logging.CRITICAL: bold_red + log_format + reset,
+        logging.DEBUG: grey + LOG_FORMAT + reset,
+        logging.INFO: grey + LOG_FORMAT + reset,
+        logging.WARNING: yellow + LOG_FORMAT + reset,
+        logging.ERROR: red + LOG_FORMAT + reset,
+        logging.CRITICAL: bold_red + LOG_FORMAT + reset,
     }
 
     def format(self, record: logging.LogRecord) -> str:
@@ -112,12 +121,52 @@ class CustomFormatter(logging.Formatter):
         return formatter.format(record)
 
 
-def enable_logging(level: int = logging.INFO) -> None:
-    """Enable logging to send to stdout and color different levels"""
+def configure_logging(
+    force_debug: bool = False,
+    ignore_warnings: bool = False,
+    write_to_stdout: bool = False,
+    write_to_file: bool = True,
+    log_limit: int = c.DEFAULT_LOG_LIMIT,
+    log_file_dir: t.Optional[t.Union[str, Path]] = None,
+) -> None:
     logger = logging.getLogger()
+    debug = force_debug or debug_mode_enabled()
+
+    # base logger needs to be the lowest level that we plan to log
+    level = logging.DEBUG if debug else logging.INFO
     logger.setLevel(level)
-    if not logger.hasHandlers():
-        handler = logging.StreamHandler(sys.stdout)
-        handler.setLevel(level)
-        handler.setFormatter(CustomFormatter())
-        logger.addHandler(handler)
+
+    stdout_handler = logging.StreamHandler(sys.stdout)
+    stdout_handler.setFormatter(CustomFormatter())
+    stdout_handler.setLevel(
+        level if write_to_stdout else (logging.ERROR if ignore_warnings else logging.WARNING)
+    )
+    logger.addHandler(stdout_handler)
+
+    log_file_dir = log_file_dir or c.DEFAULT_LOG_FILE_DIR
+    log_path_prefix = Path(log_file_dir) / LOG_FILENAME_PREFIX
+    if write_to_file:
+        os.makedirs(str(log_file_dir), exist_ok=True)
+        filename = f"{log_path_prefix}{datetime.now().strftime('%Y_%m_%d_%H_%M_%S')}.log"
+        file_handler = logging.FileHandler(filename, mode="w", encoding="utf-8")
+        # the log files should always log at least info so that users will always have
+        # minimal info for debugging even if they specify "ignore_warnings"
+        file_handler.setLevel(level)
+        file_handler.setFormatter(logging.Formatter(LOG_FORMAT))
+        logger.addHandler(file_handler)
+
+    if log_limit > 0:
+        for path in list(sorted(glob.glob(f"{log_path_prefix}*.log"), reverse=True))[log_limit:]:
+            os.remove(path)
+
+    if debug:
+        import faulthandler
+        import signal
+
+        enable_debug_mode()
+
+        # Enable threadumps.
+        faulthandler.enable()
+        # Windows doesn't support register so we check for it here
+        if hasattr(faulthandler, "register"):
+            faulthandler.register(signal.SIGUSR1.value)
