@@ -4,10 +4,12 @@ import os
 
 import click
 
-path = click.option(
-    "--path",
-    default=os.getcwd(),
-    help="Path to the models directory.",
+paths = click.option(
+    "-p",
+    "--paths",
+    multiple=True,
+    default=[os.getcwd()],
+    help="Path(s) to the SQLMesh config/project.",
 )
 
 config = click.option(
@@ -29,10 +31,9 @@ end_time = click.option(
     help="The end datetime of the interval for which this command will be applied.",
 )
 
-latest_time = click.option(
-    "-l",
-    "--latest",
-    help="The latest time used for non incremental datasets (defaults to now).",
+execution_time = click.option(
+    "--execution-time",
+    help="The execution time (defaults to now).",
 )
 
 expand = click.option(
@@ -41,11 +42,6 @@ expand = click.option(
     help="Whether or not to expand materialized models (defaults to False). If True, all referenced models are expanded as raw queries. Multiple model names can also be specified, in which case only they will be expanded as raw queries.",
 )
 
-file = click.option(
-    "--file",
-    help="The file to which the dag image should be written.",
-)
-
 match_pattern = click.option(
     "-k",
     multiple=True,
