@@ -1,23 +1,32 @@
 from __future__ import annotations
 
 import abc
+import logging
 import sys
 import typing as t
 
-from pydantic import Field, root_validator
+from pydantic import Field
 from requests import Session
 
 from sqlmesh.core.config.base import BaseConfig
 from sqlmesh.core.config.common import concurrent_tasks_validator
 from sqlmesh.core.console import Console
-from sqlmesh.core.plan import AirflowPlanEvaluator, BuiltInPlanEvaluator, PlanEvaluator
-from sqlmesh.core.state_sync import EngineAdapterStateSync, StateReader, StateSync
+from sqlmesh.core.plan import (
+    AirflowPlanEvaluator,
+    BuiltInPlanEvaluator,
+    MWAAPlanEvaluator,
+    PlanEvaluator,
+)
+from sqlmesh.core.state_sync import EngineAdapterStateSync, StateSync
 from sqlmesh.schedulers.airflow.client import AirflowClient
+from sqlmesh.schedulers.airflow.mwaa_client import MWAAClient
+from sqlmesh.utils.errors import ConfigError
+from sqlmesh.utils.pydantic import model_validator, model_validator_v1_args
 
 if t.TYPE_CHECKING:
     from google.auth.transport.requests import AuthorizedSession
 
-    from sqlmesh.core.context import Context
+    from sqlmesh.core.context import GenericContext
 
 if sys.version_info >= (3, 9):
     from typing import Annotated, Literal
@@ -25,18 +34,22 @@ else:
     from typing_extensions import Annotated, Literal
 
 
+logger = logging.getLogger(__name__)
+
+
 class _SchedulerConfig(abc.ABC):
     """Abstract base class for Scheduler configurations."""
 
     @abc.abstractmethod
-    def create_plan_evaluator(self, context: Context) -> PlanEvaluator:
+    def create_plan_evaluator(self, context: GenericContext) -> PlanEvaluator:
         """Creates a Plan Evaluator instance.
 
         Args:
             context: The SQLMesh Context.
         """
 
-    def create_state_sync(self, context: Context) -> t.Optional[StateSync]:
+    @abc.abstractmethod
+    def create_state_sync(self, context: GenericContext) -> StateSync:
         """Creates a State Sync instance.
 
         Args:
@@ -45,41 +58,53 @@ class _SchedulerConfig(abc.ABC):
         Returns:
             The StateSync instance.
         """
-        return None
 
-    def create_state_reader(self, context: Context) -> t.Optional[StateReader]:
-        """Creates a State Reader instance.
-
-        Functionality related to evaluation on a client side (Context.evaluate, Context.run, etc.)
-        will be unavailable if a State Reader instance is available but a State Sync instance is not.
+    @abc.abstractmethod
+    def get_default_catalog(self, context: GenericContext) -> t.Optional[str]:
+        """Returns the default catalog for the Scheduler.
 
         Args:
             context: The SQLMesh Context.
-
-        Returns:
-            The StateReader instance.
         """
-        return None
 
 
-class BuiltInSchedulerConfig(_SchedulerConfig, BaseConfig):
+class _EngineAdapterStateSyncSchedulerConfig(_SchedulerConfig):
+    def create_state_sync(self, context: GenericContext) -> StateSync:
+        state_connection = (
+            context.config.get_state_connection(context.gateway) or context._connection_config
+        )
+        engine_adapter = state_connection.create_engine_adapter()
+        if not engine_adapter.SUPPORTS_ROW_LEVEL_OP:
+            raise ConfigError(
+                f"The {engine_adapter.DIALECT.upper()} engine cannot be used to store SQLMesh state - please specify a different `state_connection` engine."
+                + " See https://sqlmesh.readthedocs.io/en/stable/reference/configuration/#gateways for more information."
+            )
+        schema = context.config.get_state_schema(context.gateway)
+        return EngineAdapterStateSync(
+            engine_adapter, schema=schema, context_path=context.path, console=context.console
+        )
+
+
+class BuiltInSchedulerConfig(_EngineAdapterStateSyncSchedulerConfig, BaseConfig):
     """The Built-In Scheduler configuration."""
 
     type_: Literal["builtin"] = Field(alias="type", default="builtin")
 
-    def create_state_sync(self, context: Context) -> t.Optional[StateSync]:
-        return EngineAdapterStateSync(context.engine_adapter, context.physical_schema)
-
-    def create_plan_evaluator(self, context: Context) -> PlanEvaluator:
+    def create_plan_evaluator(self, context: GenericContext) -> PlanEvaluator:
         return BuiltInPlanEvaluator(
             state_sync=context.state_sync,
             snapshot_evaluator=context.snapshot_evaluator,
+            default_catalog=self.get_default_catalog(context),
             backfill_concurrent_tasks=context.concurrent_tasks,
             console=context.console,
+            notification_target_manager=context.notification_target_manager,
         )
 
+    def get_default_catalog(self, context: GenericContext) -> t.Optional[str]:
+        return context.engine_adapter.default_catalog
 
-class _BaseAirflowSchedulerConfig(_SchedulerConfig):
+
+class _BaseAirflowSchedulerConfig(_EngineAdapterStateSyncSchedulerConfig):
     dag_run_poll_interval_secs: int
     dag_creation_poll_interval_secs: int
     dag_creation_max_retry_attempts: int
@@ -87,20 +112,27 @@ class _BaseAirflowSchedulerConfig(_SchedulerConfig):
     backfill_concurrent_tasks: int
     ddl_concurrent_tasks: int
 
+    use_state_connection: bool
+
+    default_catalog_override: t.Optional[str]
+
     @abc.abstractmethod
     def get_client(self, console: t.Optional[Console] = None) -> AirflowClient:
         """Constructs the Airflow Client instance."""
 
-    def create_state_reader(self, context: Context) -> t.Optional[StateReader]:
-        from sqlmesh.schedulers.airflow.state_sync import HttpStateReader
+    def create_state_sync(self, context: GenericContext) -> StateSync:
+        if self.use_state_connection:
+            return super().create_state_sync(context)
 
-        return HttpStateReader(
+        from sqlmesh.schedulers.airflow.state_sync import HttpStateSync
+
+        return HttpStateSync(
             client=self.get_client(context.console),
             dag_run_poll_interval_secs=self.dag_run_poll_interval_secs,
             console=context.console,
         )
 
-    def create_plan_evaluator(self, context: Context) -> PlanEvaluator:
+    def create_plan_evaluator(self, context: GenericContext) -> PlanEvaluator:
         return AirflowPlanEvaluator(
             airflow_client=self.get_client(context.console),
             dag_run_poll_interval_secs=self.dag_run_poll_interval_secs,
@@ -111,8 +143,14 @@ class _BaseAirflowSchedulerConfig(_SchedulerConfig):
             backfill_concurrent_tasks=self.backfill_concurrent_tasks,
             ddl_concurrent_tasks=self.ddl_concurrent_tasks,
             users=context.users,
+            state_sync=context.state_sync if self.use_state_connection else None,
         )
 
+    def get_default_catalog(self, context: GenericContext) -> t.Optional[str]:
+        # The default catalog must still be set on the Airflow side.
+        default_catalog = self.get_client(context.console).default_catalog
+        return self.default_catalog_override or default_catalog
+
 
 class AirflowSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig):
     """The Airflow Scheduler configuration.
@@ -127,6 +165,10 @@ class AirflowSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig):
             whether a DAG has been created.
         backfill_concurrent_tasks: The number of concurrent tasks used for model backfilling during plan application.
         ddl_concurrent_tasks: The number of concurrent tasks used for DDL operations (table / view creation, deletion, etc).
+        max_snapshot_ids_per_request: The maximum number of snapshot IDs that can be sent in a single HTTP GET request to the Airflow Webserver.
+        use_state_connection: Whether to use the `state_connection` configuration to access the SQLMesh state.
+        default_catalog_override: Overrides the default catalog value for this project. If specified, this value takes precedence
+            over the default catalog value set on the Airflow side.
     """
 
     airflow_url: str = "http://localhost:8080/"
@@ -139,6 +181,11 @@ class AirflowSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig):
     backfill_concurrent_tasks: int = 4
     ddl_concurrent_tasks: int = 4
 
+    max_snapshot_ids_per_request: t.Optional[int] = None
+    use_state_connection: bool = False
+
+    default_catalog_override: t.Optional[str] = None
+
     type_: Literal["airflow"] = Field(alias="type", default="airflow")
 
     _concurrent_tasks_validator = concurrent_tasks_validator
@@ -152,10 +199,11 @@ class AirflowSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig):
             session=session,
             airflow_url=self.airflow_url,
             console=console,
+            snapshot_ids_batch_size=self.max_snapshot_ids_per_request,
         )
 
 
-class CloudComposerSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig):
+class CloudComposerSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig, extra="allow"):
     """The Google Cloud Composer configuration.
 
     Args:
@@ -166,6 +214,10 @@ class CloudComposerSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig):
             whether a DAG has been created.
         backfill_concurrent_tasks: The number of concurrent tasks used for model backfilling during plan application.
         ddl_concurrent_tasks: The number of concurrent tasks used for DDL operations (table / view creation, deletion, etc).
+        max_snapshot_ids_per_request: The maximum number of snapshot IDs that can be sent in a single HTTP GET request to the Airflow Webserver.
+        use_state_connection: Whether to use the `state_connection` configuration to access the SQLMesh state.
+        default_catalog_override: Overrides the default catalog value for this project. If specified, this value takes precedence
+            over the default catalog value set on the Airflow side.
     """
 
     airflow_url: str
@@ -176,14 +228,15 @@ class CloudComposerSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig):
     backfill_concurrent_tasks: int = 4
     ddl_concurrent_tasks: int = 4
 
+    max_snapshot_ids_per_request: t.Optional[int] = 20
+    use_state_connection: bool = False
+
+    default_catalog_override: t.Optional[str] = None
+
     type_: Literal["cloud_composer"] = Field(alias="type", default="cloud_composer")
 
     _concurrent_tasks_validator = concurrent_tasks_validator
 
-    class Config:
-        # See `check_supported_fields` for the supported extra fields
-        extra = "allow"
-
     def __init__(self, **data: t.Any) -> None:
         super().__init__(**data)
         self._session: t.Optional[AuthorizedSession] = data.get("session")
@@ -205,11 +258,13 @@ class CloudComposerSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig):
             airflow_url=self.airflow_url,
             session=self.session,
             console=console,
+            snapshot_ids_batch_size=self.max_snapshot_ids_per_request,
         )
 
-    @root_validator(pre=True)
+    @model_validator(mode="before")
+    @model_validator_v1_args
     def check_supported_fields(cls, values: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
-        allowed_field_names = {field.alias for field in cls.__fields__.values()}
+        allowed_field_names = {field.alias or name for name, field in cls.all_field_infos().items()}
         allowed_field_names.add("session")
 
         for field_name in values:
@@ -218,7 +273,64 @@ class CloudComposerSchedulerConfig(_BaseAirflowSchedulerConfig, BaseConfig):
         return values
 
 
+class MWAASchedulerConfig(_EngineAdapterStateSyncSchedulerConfig, BaseConfig):
+    """The AWS MWAA Scheduler configuration.
+
+    Args:
+        environment: The name of the MWAA environment.
+        dag_run_poll_interval_secs: Determines how often a running DAG can be polled (in seconds).
+        dag_creation_poll_interval_secs: Determines how often SQLMesh should check whether a DAG has been created (in seconds).
+        dag_creation_max_retry_attempts: Determines the maximum number of attempts that SQLMesh will make while checking for
+            whether a DAG has been created.
+        backfill_concurrent_tasks: The number of concurrent tasks used for model backfilling during plan application.
+        ddl_concurrent_tasks: The number of concurrent tasks used for DDL operations (table / view creation, deletion, etc).
+        default_catalog_override: Overrides the default catalog value for this project. If specified, this value takes precedence
+            over the default catalog value set on the Airflow side.
+    """
+
+    environment: str
+    dag_run_poll_interval_secs: int = 10
+    dag_creation_poll_interval_secs: int = 30
+    dag_creation_max_retry_attempts: int = 10
+
+    backfill_concurrent_tasks: int = 4
+    ddl_concurrent_tasks: int = 4
+
+    default_catalog_override: t.Optional[str] = None
+
+    type_: Literal["mwaa"] = Field(alias="type", default="mwaa")
+
+    _concurrent_tasks_validator = concurrent_tasks_validator
+
+    def get_client(self, console: t.Optional[Console] = None) -> MWAAClient:
+        return MWAAClient(self.environment, console=console)
+
+    def create_plan_evaluator(self, context: GenericContext) -> PlanEvaluator:
+        return MWAAPlanEvaluator(
+            client=self.get_client(context.console),
+            state_sync=context.state_sync,
+            console=context.console,
+            dag_run_poll_interval_secs=self.dag_run_poll_interval_secs,
+            dag_creation_poll_interval_secs=self.dag_creation_poll_interval_secs,
+            dag_creation_max_retry_attempts=self.dag_creation_max_retry_attempts,
+            notification_targets=context.notification_targets,
+            backfill_concurrent_tasks=self.backfill_concurrent_tasks,
+            ddl_concurrent_tasks=self.ddl_concurrent_tasks,
+            users=context.users,
+        )
+
+    def get_default_catalog(self, context: GenericContext) -> t.Optional[str]:
+        # The default catalog must still be set on the Airflow side.
+        default_catalog = self.get_client(context.console).default_catalog
+        return self.default_catalog_override or default_catalog
+
+
 SchedulerConfig = Annotated[
-    t.Union[BuiltInSchedulerConfig, AirflowSchedulerConfig, CloudComposerSchedulerConfig],
+    t.Union[
+        BuiltInSchedulerConfig,
+        AirflowSchedulerConfig,
+        CloudComposerSchedulerConfig,
+        MWAASchedulerConfig,
+    ],
     Field(discriminator="type_"),
 ]
