@@ -1,22 +1,32 @@
-from sqlmesh.core.model.common import parse_model_name
+from sqlmesh.core.model.cache import ModelCache, OptimizedQueryCache
 from sqlmesh.core.model.decorator import model
 from sqlmesh.core.model.definition import (
     Model,
     PythonModel,
     SeedModel,
     SqlModel,
+    create_external_model,
     create_python_model,
     create_seed_model,
     create_sql_model,
-    load_model,
+    load_sql_based_model,
 )
 from sqlmesh.core.model.kind import (
+    EmbeddedKind,
+    ExternalKind,
+    FullKind,
     IncrementalByTimeRangeKind,
     IncrementalByUniqueKeyKind,
+    IncrementalUnmanagedKind,
     ModelKind,
+    ModelKindMixin,
     ModelKindName,
+    SCDType2ByColumnKind,
+    SCDType2ByTimeKind,
     SeedKind,
     TimeColumn,
+    ViewKind,
+    model_kind_validator,
 )
 from sqlmesh.core.model.meta import ModelMeta
 from sqlmesh.core.model.seed import Seed
