@@ -1,13 +1,16 @@
 import typing as t
 from enum import Enum
 
-from sqlmesh.core.environment import Environment
+from sqlmesh.core.environment import Environment, EnvironmentNamingInfo
 from sqlmesh.core.snapshot import (
+    DeployabilityIndex,
     Snapshot,
     SnapshotEvaluator,
     SnapshotId,
+    SnapshotTableCleanupTask,
     SnapshotTableInfo,
 )
+from sqlmesh.core.state_sync import cleanup_expired_views
 from sqlmesh.utils.date import TimeLike
 from sqlmesh.utils.pydantic import PydanticModel
 
@@ -32,33 +35,35 @@ class EvaluateCommandPayload(PydanticModel):
     parent_snapshots: t.Dict[str, Snapshot]
     start: TimeLike
     end: TimeLike
-    latest: TimeLike
-    is_dev: bool
+    execution_time: TimeLike
+    deployability_index: DeployabilityIndex
 
 
 class PromoteCommandPayload(PydanticModel):
-    snapshots: t.List[SnapshotTableInfo]
-    environment: str
-    is_dev: bool
+    snapshots: t.List[Snapshot]
+    environment_naming_info: EnvironmentNamingInfo
+    deployability_index: DeployabilityIndex
 
 
 class DemoteCommandPayload(PydanticModel):
     snapshots: t.List[SnapshotTableInfo]
-    environment: str
+    environment_naming_info: EnvironmentNamingInfo
 
 
 class CleanupCommandPayload(PydanticModel):
     environments: t.List[Environment]
-    snapshots: t.List[SnapshotTableInfo]
+    tasks: t.List[SnapshotTableCleanupTask]
 
 
 class CreateTablesCommandPayload(PydanticModel):
     target_snapshot_ids: t.List[SnapshotId]
     snapshots: t.List[Snapshot]
+    deployability_index: DeployabilityIndex
 
 
 class MigrateTablesCommandPayload(PydanticModel):
-    snapshots: t.List[SnapshotTableInfo]
+    target_snapshot_ids: t.List[SnapshotId]
+    snapshots: t.List[Snapshot]
 
 
 def evaluate(
@@ -70,21 +75,22 @@ def evaluate(
     parent_snapshots = command_payload.parent_snapshots
     parent_snapshots[command_payload.snapshot.name] = command_payload.snapshot
 
-    evaluator.evaluate(
+    wap_id = evaluator.evaluate(
         command_payload.snapshot,
-        command_payload.start,
-        command_payload.end,
-        command_payload.latest,
+        start=command_payload.start,
+        end=command_payload.end,
+        execution_time=command_payload.execution_time,
         snapshots=parent_snapshots,
-        is_dev=command_payload.is_dev,
+        deployability_index=command_payload.deployability_index,
     )
     evaluator.audit(
         snapshot=command_payload.snapshot,
         start=command_payload.start,
         end=command_payload.end,
-        latest=command_payload.latest,
+        execution_time=command_payload.execution_time,
         snapshots=parent_snapshots,
-        is_dev=command_payload.is_dev,
+        deployability_index=command_payload.deployability_index,
+        wap_id=wap_id,
     )
 
 
@@ -95,8 +101,8 @@ def promote(
         command_payload = PromoteCommandPayload.parse_raw(command_payload)
     evaluator.promote(
         command_payload.snapshots,
-        command_payload.environment,
-        is_dev=command_payload.is_dev,
+        command_payload.environment_naming_info,
+        deployability_index=command_payload.deployability_index,
     )
 
 
@@ -105,7 +111,10 @@ def demote(
 ) -> None:
     if isinstance(command_payload, str):
         command_payload = DemoteCommandPayload.parse_raw(command_payload)
-    evaluator.demote(command_payload.snapshots, command_payload.environment)
+    evaluator.demote(
+        command_payload.snapshots,
+        command_payload.environment_naming_info,
+    )
 
 
 def cleanup(
@@ -113,9 +122,9 @@ def cleanup(
 ) -> None:
     if isinstance(command_payload, str):
         command_payload = CleanupCommandPayload.parse_raw(command_payload)
-    for environment in command_payload.environments:
-        evaluator.demote(environment.snapshots, environment.name)
-    evaluator.cleanup(command_payload.snapshots)
+
+    cleanup_expired_views(evaluator.adapter, command_payload.environments)
+    evaluator.cleanup(command_payload.tasks)
 
 
 def create_tables(
@@ -127,7 +136,11 @@ def create_tables(
 
     snapshots_by_id = {s.snapshot_id: s for s in command_payload.snapshots}
     target_snapshots = [snapshots_by_id[sid] for sid in command_payload.target_snapshot_ids]
-    evaluator.create(target_snapshots, snapshots_by_id)
+    evaluator.create(
+        target_snapshots,
+        snapshots_by_id,
+        deployability_index=command_payload.deployability_index,
+    )
 
 
 def migrate_tables(
@@ -136,7 +149,9 @@ def migrate_tables(
 ) -> None:
     if isinstance(command_payload, str):
         command_payload = MigrateTablesCommandPayload.parse_raw(command_payload)
-    evaluator.migrate(command_payload.snapshots)
+    snapshots_by_id = {s.snapshot_id: s for s in command_payload.snapshots}
+    target_snapshots = [snapshots_by_id[sid] for sid in command_payload.target_snapshot_ids]
+    evaluator.migrate(target_snapshots, snapshots_by_id)
 
 
 COMMAND_HANDLERS: t.Dict[CommandType, t.Callable[[SnapshotEvaluator, str], None]] = {
