@@ -3,19 +3,21 @@ from __future__ import annotations
 import typing as t
 
 from sqlmesh.core import constants as c
-from sqlmesh.core._typing import NotificationTarget
 from sqlmesh.core.environment import Environment
+from sqlmesh.core.notification_target import NotificationTarget
 from sqlmesh.core.scheduler import Interval
 from sqlmesh.core.snapshot import (
+    DeployabilityIndex,
     Snapshot,
     SnapshotId,
-    SnapshotIdLike,
     SnapshotInfoLike,
+    SnapshotIntervals,
     SnapshotTableInfo,
 )
+from sqlmesh.core.snapshot.definition import Interval as SnapshotInterval
 from sqlmesh.core.user import User
+from sqlmesh.utils import sanitize_name
 from sqlmesh.utils.date import TimeLike
-from sqlmesh.utils.errors import SQLMeshError
 from sqlmesh.utils.pydantic import PydanticModel
 
 JANITOR_DAG_ID = "sqlmesh_janitor_dag"
@@ -27,10 +29,7 @@ PLAN_AIRFLOW_TAG = "sqlmesh_plan"
 
 SNAPSHOT_CLEANUP_COMMAND_XCOM_KEY = "snapshot_cleanup_command"
 
-PLAN_DAG_SPEC_KEY_PREFIX = "sqlmesh__plan_dag_spec"
-SNAPSHOT_PAYLOAD_KEY_PREFIX = "sqlmesh__snapshot_payload"
-SNAPSHOT_VERSION_KEY_PREFIX = "sqlmesh__snapshot_version_index"
-ENV_KEY_PREFIX = "sqlmesh__environment"
+DEFAULT_CATALOG_VARIABLE_NAME = "sqlmesh_default_catalog"
 
 AIRFLOW_LOCAL_URL = "http://localhost:8080/"
 
@@ -43,38 +42,53 @@ class PlanApplicationRequest(PydanticModel):
     environment: Environment
     no_gaps: bool
     skip_backfill: bool
-    restatements: t.Set[str]
+    restatements: t.Dict[str, SnapshotInterval]
     notification_targets: t.List[NotificationTarget]
     backfill_concurrent_tasks: int
     ddl_concurrent_tasks: int
     users: t.List[User]
     is_dev: bool
+    forward_only: bool
+    models_to_backfill: t.Optional[t.Set[str]] = None
+    end_bounded: bool
+    ensure_finalized_snapshots: bool
+    directly_modified_snapshots: t.List[SnapshotId]
+    indirectly_modified_snapshots: t.Dict[str, t.List[SnapshotId]]
+    removed_snapshots: t.List[SnapshotId]
+
+    def is_selected_for_backfill(self, model_fqn: str) -> bool:
+        return self.models_to_backfill is None or model_fqn in self.models_to_backfill
 
 
 class BackfillIntervalsPerSnapshot(PydanticModel):
     snapshot_id: SnapshotId
     intervals: t.List[Interval]
+    before_promote: bool = True
 
 
 class PlanDagSpec(PydanticModel):
     request_id: str
-    environment_name: str
+    environment: Environment
     new_snapshots: t.List[Snapshot]
     backfill_intervals_per_snapshot: t.List[BackfillIntervalsPerSnapshot]
-    promoted_snapshots: t.List[SnapshotTableInfo]
     demoted_snapshots: t.List[SnapshotTableInfo]
-    start: TimeLike
-    end: t.Optional[TimeLike]
-    unpaused_dt: t.Optional[TimeLike]
+    unpaused_dt: t.Optional[TimeLike] = None
     no_gaps: bool
-    plan_id: str
-    previous_plan_id: t.Optional[str]
     notification_targets: t.List[NotificationTarget]
     backfill_concurrent_tasks: int
     ddl_concurrent_tasks: int
     users: t.List[User]
     is_dev: bool
-    environment_expiration_ts: t.Optional[int]
+    forward_only: t.Optional[bool] = None
+    dag_start_ts: t.Optional[int] = None
+    deployability_index: DeployabilityIndex = DeployabilityIndex.all_deployable()
+    deployability_index_for_creation: DeployabilityIndex = DeployabilityIndex.all_deployable()
+    no_gaps_snapshot_names: t.Optional[t.Set[str]] = None
+    models_to_backfill: t.Optional[t.Set[str]] = None
+    ensure_finalized_snapshots: bool = False
+    directly_modified_snapshots: t.Optional[t.List[SnapshotId]] = None
+    indirectly_modified_snapshots: t.Optional[t.Dict[str, t.List[SnapshotId]]] = None
+    removed_snapshots: t.Optional[t.List[SnapshotId]] = None
 
 
 class EnvironmentsResponse(PydanticModel):
@@ -85,26 +99,25 @@ class SnapshotsResponse(PydanticModel):
     snapshots: t.List[Snapshot]
 
 
+class SnapshotIntervalsResponse(PydanticModel):
+    snapshot_intervals: t.List[SnapshotIntervals]
+
+
 class SnapshotIdsResponse(PydanticModel):
     snapshot_ids: t.List[SnapshotId]
 
 
-def snapshot_key(snapshot: SnapshotIdLike) -> str:
-    return snapshot_key_from_name_identifier(snapshot.name, snapshot.identifier)
+class ExistingModelsResponse(PydanticModel):
+    names: t.List[str]
 
 
-def snapshot_key_from_name_identifier(name: str, identifier: str) -> str:
-    return f"{SNAPSHOT_PAYLOAD_KEY_PREFIX}__{name}__{identifier}"
+class InvalidateEnvironmentResponse(PydanticModel):
+    name: str
 
 
-def snapshot_version_key(name: str, version: t.Optional[str] = None) -> str:
-    if not version:
-        raise SQLMeshError("Version cannot be empty")
-    return f"{SNAPSHOT_VERSION_KEY_PREFIX}__{name}__{version}"
-
-
-def name_from_snapshot_version_key(key: str) -> str:
-    return key[len(f"{SNAPSHOT_VERSION_KEY_PREFIX}__") : key.rindex("__")]
+class IntervalEndResponse(PydanticModel):
+    environment: str
+    max_interval_end: t.Optional[int] = None
 
 
 def dag_id_for_snapshot_info(info: SnapshotInfoLike) -> str:
@@ -113,21 +126,8 @@ def dag_id_for_snapshot_info(info: SnapshotInfoLike) -> str:
 
 
 def dag_id_for_name_version(name: str, version: str) -> str:
-    return f"sqlmesh_snapshot_{name}_{version}_dag"
+    return f"sqlmesh_snapshot_{sanitize_name(name)}_{version}_dag"
 
 
 def plan_application_dag_id(environment: str, request_id: str) -> str:
     return f"sqlmesh_plan_application__{environment}__{request_id}"
-
-
-def environment_key(env: str) -> str:
-    return f"{ENV_KEY_PREFIX}__{env}"
-
-
-def plan_dag_spec_key(request_id: str) -> str:
-    return f"{PLAN_DAG_SPEC_KEY_PREFIX}__{request_id}"
-
-
-def plan_dag_spec_key_from_dag_id(dag_id: str) -> str:
-    request_id = dag_id[dag_id.rindex("__") + 2 :]
-    return plan_dag_spec_key(request_id)
