@@ -1,20 +1,27 @@
 from __future__ import annotations
 
 import importlib
+import json
+import re
 import typing as t
 from collections import defaultdict
+from enum import Enum
 
 from jinja2 import Environment, Template, nodes
-from pydantic import validator
-from sqlglot import Dialect, Parser, TokenType
+from sqlglot import Dialect, Expression, Parser, TokenType
 
+from sqlmesh.core import constants as c
+from sqlmesh.core import dialect as d
 from sqlmesh.utils import AttributeDict
-from sqlmesh.utils.pydantic import PydanticModel
+from sqlmesh.utils.pydantic import PydanticModel, field_serializer, field_validator
+
+SQLMESH_JINJA_PACKAGE = "sqlmesh.utils.jinja"
 
 
 def environment(**kwargs: t.Any) -> Environment:
     extensions = kwargs.pop("extensions", [])
     extensions.append("jinja2.ext.do")
+    extensions.append("jinja2.ext.loopcontrols")
     return Environment(extensions=extensions, **kwargs)
 
 
@@ -22,7 +29,7 @@ ENVIRONMENT = environment()
 
 
 class MacroReference(PydanticModel, frozen=True):
-    package: t.Optional[str]
+    package: t.Optional[str] = None
     name: str
 
     @property
@@ -47,18 +54,6 @@ class MacroReturnVal(Exception):
         self.value = val
 
 
-def macro_return(macro: t.Callable) -> t.Callable:
-    """Decorator to pass data back to the caller"""
-
-    def wrapper(*args: t.Any, **kwargs: t.Any) -> t.Any:
-        try:
-            return macro(*args, **kwargs)
-        except MacroReturnVal as ret:
-            return ret.value
-
-    return wrapper
-
-
 class MacroExtractor(Parser):
     def extract(self, jinja: str, dialect: str = "") -> t.Dict[str, MacroInfo]:
         """Extract a dictionary of macro definitions from a jinja string.
@@ -72,7 +67,7 @@ class MacroExtractor(Parser):
         """
         self.reset()
         self.sql = jinja
-        self._tokens = Dialect.get_or_raise(dialect)().tokenizer.tokenize(jinja)
+        self._tokens = Dialect.get_or_raise(dialect).tokenizer.tokenize(jinja)
         self._index = -1
         self._advance()
 
@@ -92,7 +87,7 @@ class MacroExtractor(Parser):
                 macro_str = self._find_sql(macro_start, self._next)
                 macros[name] = MacroInfo(
                     definition=macro_str,
-                    depends_on=list(extract_macro_references(macro_str)),
+                    depends_on=list(extract_macro_references_and_variables(macro_str)[0]),
                 )
 
             self._advance()
@@ -124,7 +119,9 @@ def render_jinja(query: str, methods: t.Optional[t.Dict[str, t.Any]] = None) ->
     return ENVIRONMENT.from_string(query).render(methods or {})
 
 
-def find_call_names(node: nodes.Node, vars_in_scope: t.Set[str]) -> t.Iterator[t.Tuple[str, ...]]:
+def find_call_names(
+    node: nodes.Node, vars_in_scope: t.Set[str]
+) -> t.Iterator[t.Tuple[t.Tuple[str, ...], nodes.Call]]:
     vars_in_scope = vars_in_scope.copy()
     for child_node in node.iter_child_nodes():
         if "target" in child_node.fields:
@@ -141,22 +138,32 @@ def find_call_names(node: nodes.Node, vars_in_scope: t.Set[str]) -> t.Iterator[t
         elif isinstance(child_node, nodes.Call):
             name = call_name(child_node)
             if name[0][0] != "'" and name[0] not in vars_in_scope:
-                yield name
+                yield (name, child_node)
         yield from find_call_names(child_node, vars_in_scope)
 
 
-def extract_call_names(jinja_str: str) -> t.List[t.Tuple[str, ...]]:
+def extract_call_names(jinja_str: str) -> t.List[t.Tuple[t.Tuple[str, ...], nodes.Call]]:
     return list(find_call_names(ENVIRONMENT.parse(jinja_str), set()))
 
 
-def extract_macro_references(jinja_str: str) -> t.Set[MacroReference]:
-    result = set()
-    for call_name in extract_call_names(jinja_str):
-        if len(call_name) == 1:
-            result.add(MacroReference(name=call_name[0]))
-        elif len(call_name) == 2:
-            result.add(MacroReference(package=call_name[0], name=call_name[1]))
-    return result
+def extract_macro_references_and_variables(
+    *jinja_strs: str,
+) -> t.Tuple[t.Set[MacroReference], t.Set[str]]:
+    macro_references = set()
+    variables = set()
+    for jinja_str in jinja_strs:
+        for call_name, node in extract_call_names(jinja_str):
+            if call_name[0] == c.VAR:
+                args = [jinja_call_arg_name(arg) for arg in node.args]
+                if args and args[0]:
+                    variables.add(args[0].lower())
+            elif call_name[0] == c.GATEWAY:
+                variables.add(c.GATEWAY)
+            elif len(call_name) == 1:
+                macro_references.add(MacroReference(name=call_name[0]))
+            elif len(call_name) == 2:
+                macro_references.add(MacroReference(package=call_name[0], name=call_name[1]))
+    return macro_references, variables
 
 
 JinjaGlobalAttribute = t.Union[str, int, float, bool, AttributeDict]
@@ -171,21 +178,49 @@ class JinjaMacroRegistry(PydanticModel):
         global_objs: The global objects.
         create_builtins_module: The name of a module which defines the `create_builtins` factory
             function that will be used to construct builtin variables and functions.
+        root_package_name: The name of the root package. If specified root macros will be available
+            as both `root_package_name.macro_name` and `macro_name`.
+        top_level_packages: The list of top-level packages. Macros in this packages will be available
+            as both `package_name.macro_name` and `macro_name`.
     """
 
     packages: t.Dict[str, t.Dict[str, MacroInfo]] = {}
     root_macros: t.Dict[str, MacroInfo] = {}
     global_objs: t.Dict[str, JinjaGlobalAttribute] = {}
-    create_builtins_module: t.Optional[str] = None
+    create_builtins_module: t.Optional[str] = SQLMESH_JINJA_PACKAGE
+    root_package_name: t.Optional[str] = None
+    top_level_packages: t.List[str] = []
 
     _parser_cache: t.Dict[t.Tuple[t.Optional[str], str], Template] = {}
     __environment: t.Optional[Environment] = None
 
-    @validator("global_objs", pre=True)
-    def _validate_attribute_dict(cls, value: t.Any) -> t.Any:
-        if isinstance(value, t.Dict):
-            return {k: AttributeDict(v) if isinstance(v, dict) else v for k, v in value.items()}
-        return value
+    @field_validator("global_objs", mode="before")
+    @classmethod
+    def _validate_global_objs(cls, value: t.Any) -> t.Any:
+        def _normalize(val: t.Any) -> t.Any:
+            if isinstance(val, dict):
+                return AttributeDict({k: _normalize(v) for k, v in val.items()})
+            if isinstance(val, list):
+                return [_normalize(v) for v in val]
+            if isinstance(val, set):
+                return [_normalize(v) for v in sorted(val)]
+            if isinstance(val, Enum):
+                return val.value
+            return val
+
+        return _normalize(value)
+
+    @field_serializer("global_objs")
+    def _serialize_attribute_dict(
+        self, value: t.Dict[str, JinjaGlobalAttribute]
+    ) -> t.Dict[str, t.Any]:
+        # NOTE: This is called only when used with Pydantic V2.
+        def _convert(
+            val: t.Union[t.Dict[str, JinjaGlobalAttribute], t.Dict[str, t.Any]]
+        ) -> t.Dict[str, t.Any]:
+            return {k: _convert(v) if isinstance(v, AttributeDict) else v for k, v in val.items()}
+
+        return _convert(value)
 
     def add_macros(self, macros: t.Dict[str, MacroInfo], package: t.Optional[str] = None) -> None:
         """Adds macros to the target package.
@@ -203,6 +238,14 @@ class JinjaMacroRegistry(PydanticModel):
         else:
             self.root_macros.update(macros)
 
+    def add_globals(self, globals: t.Dict[str, JinjaGlobalAttribute]) -> None:
+        """Adds global objects to the registry.
+
+        Args:
+            globals: The global objects that should be added.
+        """
+        self.global_objs.update(**self._validate_global_objs(globals))
+
     def build_macro(self, reference: MacroReference, **kwargs: t.Any) -> t.Optional[t.Callable]:
         """Builds a Python callable for a macro with the given reference.
 
@@ -211,63 +254,78 @@ class JinjaMacroRegistry(PydanticModel):
         Returns:
             The macro as a Python callable or None if not found.
         """
-        if reference.package is not None and reference.name not in self.packages.get(
-            reference.package, {}
-        ):
-            return None
-        if reference.package is None and reference.name not in self.root_macros:
-            return None
-
-        global_vars = self._create_builtin_globals(kwargs)
-        return self._make_callable(reference.name, reference.package, {}, global_vars)
+        env: Environment = self.build_environment(**kwargs)
+        if reference.package is not None:
+            package = env.globals.get(reference.package, {})
+            return package.get(reference.name)  # type: ignore
+        return env.globals.get(reference.name)  # type: ignore
 
     def build_environment(self, **kwargs: t.Any) -> Environment:
         """Builds a new Jinja environment based on this registry."""
 
-        global_vars = self._create_builtin_globals(kwargs)
-
-        callable_cache: t.Dict[t.Tuple[t.Optional[str], str], t.Callable] = {}
+        context: t.Dict[str, t.Any] = {}
 
         root_macros = {
-            name: self._make_callable(name, None, callable_cache, global_vars)
+            name: self._MacroWrapper(name, None, self, context)
             for name, macro in self.root_macros.items()
-            if not _is_private_macro(name)
         }
 
         package_macros: t.Dict[str, t.Any] = defaultdict(AttributeDict)
         for package_name, macros in self.packages.items():
-            for macro_name, macro in macros.items():
-                if not _is_private_macro(macro_name):
-                    package_macros[package_name][macro_name] = self._make_callable(
-                        macro_name, package_name, callable_cache, global_vars
-                    )
+            for macro_name in macros:
+                package_macros[package_name][macro_name] = self._MacroWrapper(
+                    macro_name, package_name, self, context
+                )
+
+        if self.root_package_name is not None:
+            package_macros[self.root_package_name].update(root_macros)
 
         env = environment()
-        env.globals.update(
-            {
-                **root_macros,
-                **package_macros,
-                **global_vars,
-            }
-        )
+
+        builtin_globals = self._create_builtin_globals(kwargs)
+        for top_level_package_name in self.top_level_packages:
+            # Make sure that the top-level package doesn't fully override the same builtin package.
+            package_macros[top_level_package_name] = AttributeDict(
+                {
+                    **(builtin_globals.pop(top_level_package_name, None) or {}),
+                    **(package_macros.get(top_level_package_name) or {}),
+                }
+            )
+            root_macros.update(package_macros[top_level_package_name])
+
+        context.update(builtin_globals)
+        context.update(root_macros)
+        context.update(package_macros)
+
+        env.globals.update(context)
         env.filters.update(self._environment.filters)
         return env
 
-    def trim(self, dependencies: t.Iterable[MacroReference]) -> JinjaMacroRegistry:
+    def trim(
+        self, dependencies: t.Iterable[MacroReference], package: t.Optional[str] = None
+    ) -> JinjaMacroRegistry:
         """Trims the registry by keeping only macros with given references and their transitive dependencies.
 
         Args:
             dependencies: References to macros that should be kept.
+            package: The name of the package in the context of which the trimming should be performed.
 
         Returns:
             A new trimmed registry.
         """
         dependencies_by_package: t.Dict[t.Optional[str], t.Set[str]] = defaultdict(set)
         for dep in dependencies:
-            dependencies_by_package[dep.package].add(dep.name)
+            dependencies_by_package[dep.package or package].add(dep.name)
+
+        top_level_packages = self.top_level_packages.copy()
+        if package is not None:
+            top_level_packages.append(package)
 
         result = JinjaMacroRegistry(
-            global_objs=self.global_objs.copy(), create_builtins_module=self.create_builtins_module
+            global_objs=self.global_objs.copy(),
+            create_builtins_module=self.create_builtins_module,
+            root_package_name=self.root_package_name,
+            top_level_packages=top_level_packages,
         )
         for package, names in dependencies_by_package.items():
             result = result.merge(self._trim_macros(names, package))
@@ -306,55 +364,67 @@ class JinjaMacroRegistry(PydanticModel):
             root_macros=root_macros,
             global_objs=global_objs,
             create_builtins_module=self.create_builtins_module or other.create_builtins_module,
+            root_package_name=self.root_package_name or other.root_package_name,
+            top_level_packages=[*self.top_level_packages, *other.top_level_packages],
         )
 
-    def _make_callable(
-        self,
-        name: str,
-        package: t.Optional[str],
-        callable_cache: t.Dict[t.Tuple[t.Optional[str], str], t.Callable],
-        macro_vars: t.Dict[str, t.Any],
-    ) -> t.Callable:
-        cache_key = (package, name)
-        if cache_key in callable_cache:
-            return callable_cache[cache_key]
+    def to_expressions(self) -> t.List[Expression]:
+        output: t.List[Expression] = []
 
-        macro_vars = macro_vars.copy()
-        macro = self._get_macro(name, package)
-
-        package_macros: t.Dict[str, AttributeDict] = defaultdict(AttributeDict)
-        for dependency in macro.depends_on:
-            if (dependency.package is None and dependency.name == name) or not self._macro_exists(
-                dependency.name, dependency.package or package
-            ):
-                continue
-
-            upstream_callable = self._make_callable(
-                dependency.name, dependency.package or package, callable_cache, macro_vars
+        filtered_objs = {
+            k: v for k, v in self.global_objs.items() if k in ("refs", "sources", "vars")
+        }
+        if filtered_objs:
+            output.append(
+                d.PythonCode(
+                    expressions=[
+                        f"{k} = '{v}'" if isinstance(v, str) else f"{k} = {v}"
+                        for k, v in sorted(filtered_objs.items())
+                    ]
+                )
             )
-            if dependency.package is None:
-                macro_vars[dependency.name] = upstream_callable
-            else:
-                package_macros[dependency.package][dependency.name] = upstream_callable
 
-        macro_vars.update(package_macros)
+        for macro_name, macro_info in sorted(self.root_macros.items()):
+            output.append(d.jinja_statement(macro_info.definition))
 
-        template = self._parse_macro(name, package)
-        macro_callable = macro_return(
-            getattr(template.make_module(vars=macro_vars), _non_private_name(name))
-        )
-        callable_cache[cache_key] = macro_callable
-        return macro_callable
+        for _, package in sorted(self.packages.items()):
+            for macro_name, macro_info in sorted(package.items()):
+                output.append(d.jinja_statement(macro_info.definition))
+
+        return output
+
+    @property
+    def data_hash_values(self) -> t.List[str]:
+        data = []
+
+        for macro_name, macro in sorted(self.root_macros.items()):
+            data.append(macro_name)
+            data.append(macro.definition)
+
+        for _, package in sorted(self.packages.items()):
+            for macro_name, macro in sorted(package.items()):
+                data.append(macro_name)
+                data.append(macro.definition)
+
+        trimmed_global_objs = {
+            k: self.global_objs[k] for k in ("refs", "sources", "vars") if k in self.global_objs
+        }
+        data.append(json.dumps(trimmed_global_objs, sort_keys=True))
+
+        return data
+
+    def __deepcopy__(self, memo: t.Optional[t.Dict[int, t.Any]] = None) -> JinjaMacroRegistry:
+        return JinjaMacroRegistry.parse_obj(self.dict())
 
     def _parse_macro(self, name: str, package: t.Optional[str]) -> Template:
         cache_key = (package, name)
         if cache_key not in self._parser_cache:
             macro = self._get_macro(name, package)
 
-            definition: t.Union[str, nodes.Template] = macro.definition
+            definition: nodes.Template = self._environment.parse(macro.definition)
             if _is_private_macro(name):
                 # A workaround to expose private jinja macros.
-                definition = self._to_non_private_macro_def(name, macro.definition)
+                definition = self._to_non_private_macro_def(name, definition)
 
             self._parser_cache[cache_key] = self._environment.from_string(definition)
         return self._parser_cache[cache_key]
@@ -366,18 +436,27 @@ class JinjaMacroRegistry(PydanticModel):
             self.__environment.filters.update(self._create_builtin_filters())
         return self.__environment
 
-    def _trim_macros(self, names: t.Set[str], package: t.Optional[str]) -> JinjaMacroRegistry:
+    def _trim_macros(
+        self,
+        names: t.Set[str],
+        package: t.Optional[str] = None,
+        visited: t.Optional[t.Dict[t.Optional[str], t.Set[str]]] = None,
+    ) -> JinjaMacroRegistry:
+        if visited is None:
+            visited = defaultdict(set)
+
         macros = self.packages.get(package, {}) if package is not None else self.root_macros
         trimmed_macros = {}
 
         dependencies: t.Dict[t.Optional[str], t.Set[str]] = defaultdict(set)
 
         for name in names:
-            if name in macros:
+            if name in macros and name not in visited[package]:
                 macro = macros[name]
                 trimmed_macros[name] = macro
                 for dependency in macro.depends_on:
                     dependencies[dependency.package or package].add(dependency.name)
+                visited[package].add(name)
 
         if package is not None:
             result = JinjaMacroRegistry(packages={package: trimmed_macros})
@@ -385,7 +464,9 @@ class JinjaMacroRegistry(PydanticModel):
             result = JinjaMacroRegistry(root_macros=trimmed_macros)
 
         for upstream_package, upstream_names in dependencies.items():
-            result = result.merge(self._trim_macros(upstream_names, upstream_package))
+            result = result.merge(
+                self._trim_macros(upstream_names, upstream_package, visited=visited)
+            )
 
         return result
 
@@ -399,9 +480,7 @@ class JinjaMacroRegistry(PydanticModel):
     def _get_macro(self, name: str, package: t.Optional[str]) -> MacroInfo:
         return self.packages[package][name] if package is not None else self.root_macros[name]
 
-    def _to_non_private_macro_def(self, name: str, definition: str) -> nodes.Template:
-        template = self._environment.parse(definition)
-
+    def _to_non_private_macro_def(self, name: str, template: nodes.Template) -> nodes.Template:
         for node in template.find_all((nodes.Macro, nodes.Call)):
             if isinstance(node, nodes.Macro):
                 node.name = _non_private_name(name)
@@ -428,6 +507,33 @@ class JinjaMacroRegistry(PydanticModel):
                 return module.create_builtin_filters()
         return {}
 
+    class _MacroWrapper:
+        def __init__(
+            self,
+            name: str,
+            package: t.Optional[str],
+            registry: JinjaMacroRegistry,
+            context: t.Dict[str, t.Any],
+        ):
+            self.name = name
+            self.package = package
+            self.context = context
+            self.registry = registry
+
+        def __call__(self, *args: t.Any, **kwargs: t.Any) -> t.Any:
+            context = self.context.copy()
+            if self.package is not None and self.package in context:
+                context.update(context[self.package])
+
+            template = self.registry._parse_macro(self.name, self.package)
+            macro_callable = getattr(
+                template.make_module(vars=context), _non_private_name(self.name)
+            )
+            try:
+                return macro_callable(*args, **kwargs)
+            except MacroReturnVal as ret:
+                return ret.value
+
 
 def _is_private_macro(name: str) -> bool:
     return name.startswith("_")
@@ -435,3 +541,35 @@ def _is_private_macro(name: str) -> bool:
 
 def _non_private_name(name: str) -> str:
     return name.lstrip("_")
+
+
+JINJA_REGEX = re.compile(r"({{|{%)")
+
+
+def has_jinja(value: str) -> bool:
+    return JINJA_REGEX.search(value) is not None
+
+
+def jinja_call_arg_name(node: nodes.Node) -> str:
+    if isinstance(node, nodes.Const):
+        return node.value
+    return ""
+
+
+def create_var(variables: t.Dict[str, t.Any]) -> t.Callable:
+    def _var(var_name: str, default: t.Optional[t.Any] = None) -> t.Optional[t.Any]:
+        return variables.get(var_name.lower(), default)
+
+    return _var
+
+
+def create_builtin_globals(
+    jinja_macros: JinjaMacroRegistry, global_vars: t.Dict[str, t.Any], *args: t.Any, **kwargs: t.Any
+) -> t.Dict[str, t.Any]:
+    global_vars.pop(c.GATEWAY, None)
+    variables = global_vars.pop(c.SQLMESH_VARS, None) or {}
+    return {
+        c.VAR: create_var(variables),
+        c.GATEWAY: lambda: variables.get(c.GATEWAY, None),
+        **global_vars,
+    }
