@@ -20,7 +20,7 @@ from sqlmesh.utils import format_exception, unique
 from sqlmesh.utils.errors import SQLMeshError
 from sqlmesh.utils.pydantic import PydanticModel
 
-IGNORE_DECORATORS = {"hook", "macro", "model"}
+IGNORE_DECORATORS = {"macro", "model"}
 
 
 def _is_relative_to(path: t.Optional[Path | str], other: t.Optional[Path | str]) -> bool:
@@ -276,9 +276,16 @@ def build_env(
                 build_env(v, env=env, name=k, path=path)
 
     if name not in env:
-        env[name] = obj
-        if obj_module and _is_relative_to(obj_module.__file__, path):
-            walk(obj)
+        # We only need to add the undecorated code of @macro() functions in env, which
+        # is accessible through the `__wrapped__` attribute added by functools.wraps
+        env[name] = obj.__wrapped__ if getattr(obj, "__sqlmesh_macro__", None) else obj
+
+        if (
+            obj_module
+            and hasattr(obj_module, "__file__")
+            and _is_relative_to(obj_module.__file__, path)
+        ):
+            walk(env[name])
     elif env[name] != obj:
         raise SQLMeshError(
             f"Cannot store {obj} in environment, duplicate definitions found for '{name}'"
@@ -291,7 +298,6 @@ class ExecutableKind(str, Enum):
     IMPORT = "import"
     VALUE = "value"
     DEFINITION = "definition"
-    STATEMENT = "statement"
 
     def __lt__(self, other: t.Any) -> bool:
         if not isinstance(other, ExecutableKind):
@@ -299,9 +305,12 @@ class ExecutableKind(str, Enum):
         values = list(ExecutableKind.__dict__.values())
         return values.index(self) < values.index(other)
 
+    def __str__(self) -> str:
+        return self.value
+
 
 class Executable(PydanticModel):
-    payload: t.Any
+    payload: str
     kind: ExecutableKind = ExecutableKind.DEFINITION
     name: t.Optional[str] = None
     path: t.Optional[str] = None
@@ -315,14 +324,14 @@ class Executable(PydanticModel):
     def is_import(self) -> bool:
         return self.kind == ExecutableKind.IMPORT
 
-    @property
-    def is_statement(self) -> bool:
-        return self.kind == ExecutableKind.STATEMENT
-
     @property
     def is_value(self) -> bool:
         return self.kind == ExecutableKind.VALUE
 
+    @classmethod
+    def value(cls, v: t.Any) -> Executable:
+        return Executable(payload=repr(v), kind=ExecutableKind.VALUE)
+
 
 def serialize_env(env: t.Dict[str, t.Any], path: Path) -> t.Dict[str, Executable]:
     """Serializes a python function into a self contained dictionary.
@@ -346,7 +355,8 @@ def serialize_env(env: t.Dict[str, t.Any], path: Path) -> t.Dict[str, Executable
                     name=name,
                     payload=normalize_source(v),
                     kind=ExecutableKind.DEFINITION,
-                    path=str(file_path.relative_to(path.absolute())),
+                    # Do `as_posix` to serialize windows path back to POSIX
+                    path=file_path.relative_to(path.absolute()).as_posix(),
                     alias=k if name != k else None,
                 )
             else:
@@ -356,7 +366,7 @@ def serialize_env(env: t.Dict[str, t.Any], path: Path) -> t.Dict[str, Executable
                 )
         elif inspect.ismodule(v):
             name = v.__name__
-            if _is_relative_to(v.__file__, path):
+            if hasattr(v, "__file__") and _is_relative_to(v.__file__, path):
                 raise SQLMeshError(
                     f"Cannot serialize 'import {name}'. Use 'from {name} import ...' instead."
                 )
@@ -366,7 +376,7 @@ def serialize_env(env: t.Dict[str, t.Any], path: Path) -> t.Dict[str, Executable
                 kind=ExecutableKind.IMPORT,
             )
         else:
-            serialized[k] = Executable(payload=repr(v), kind=ExecutableKind.VALUE)
+            serialized[k] = Executable.value(v)
 
     return serialized
 
@@ -419,7 +429,7 @@ def print_exception(
     tb: t.List[str] = []
 
     for error_line in format_exception(exception):
-        match = re.search(f'File "<string>", line (.*), in (.*)', error_line)
+        match = re.search('File "<string>", line (.*), in (.*)', error_line)
 
         if not match:
             tb.append(error_line)
@@ -465,9 +475,9 @@ def print_exception(
 
 
 def import_python_file(path: Path, relative_base: Path = Path()) -> types.ModuleType:
-    module_name = str(
-        path.absolute().relative_to(relative_base.absolute()).with_suffix("")
-    ).replace(os.path.sep, ".")
+    relative_path = path.absolute().relative_to(relative_base.absolute())
+    module_name = str(relative_path.with_suffix("")).replace(os.path.sep, ".")
+
     # remove the entire module hierarchy in case they were already loaded
     parts = module_name.split(".")
     for i in range(len(parts)):
