@@ -1,8 +1,14 @@
+import typing as t
+
+import pandas as pd
 import pytest
 from sqlglot import expressions as exp
 from sqlglot import parse_one
 
 from sqlmesh.core.engine_adapter import DuckDBEngineAdapter, EngineAdapter
+from tests.core.engine_adapter import to_sql_calls
+
+pytestmark = [pytest.mark.duckdb, pytest.mark.engine]
 
 
 @pytest.fixture
@@ -21,13 +27,12 @@ def test_create_view(adapter: EngineAdapter, duck_conn):
 
 
 def test_create_schema(adapter: EngineAdapter, duck_conn):
-    adapter.create_schema("test_schema")
     adapter.create_schema("test_schema")
     assert duck_conn.execute(
         "SELECT 1 FROM information_schema.schemata WHERE schema_name = 'test_schema'"
     ).fetchall() == [(1,)]
     with pytest.raises(Exception):
-        adapter.create_schema("test_schema", ignore_if_exists=False)
+        adapter.create_schema("test_schema", ignore_if_exists=False, warn_on_error=False)
 
 
 def test_table_exists(adapter: EngineAdapter, duck_conn):
@@ -50,22 +55,23 @@ def test_create_table(adapter: EngineAdapter, duck_conn):
         "test_table2",
         columns_to_types,
         storage_format="ICEBERG",
-        partitioned_by=["colb"],
+        partitioned_by=[exp.to_column("colb")],
     )
     assert duck_conn.execute("DESCRIBE test_table").fetchall() == expected_columns
 
 
-def test_transaction(adapter: EngineAdapter, duck_conn):
-    adapter.create_table("test_table", {"a": exp.DataType.build("int")})
-    with adapter.transaction():
-        adapter.execute("INSERT INTO test_table (a) VALUES (1)")
-    assert duck_conn.execute("SELECT * FROM test_table").fetchall() == [(1,)]
+def test_replace_query_pandas(adapter: EngineAdapter, duck_conn):
+    df = pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]})
+    adapter.replace_query(
+        "test_table", df, {"a": exp.DataType.build("long"), "b": exp.DataType.build("long")}
+    )
+    pd.testing.assert_frame_equal(adapter.fetchdf("SELECT * FROM test_table"), df)
 
-    # Assert transaction was rolled back if an exception was raised
-    try:
-        with adapter.transaction():
-            adapter.execute("INSERT INTO test_table (a) VALUES (1)")
-            raise Exception
-    except Exception:
-        pass
-    assert duck_conn.execute("SELECT * FROM test_table").fetchall() == [(1,)]
+
+def test_set_current_catalog(make_mocked_engine_adapter: t.Callable, duck_conn):
+    adapter = make_mocked_engine_adapter(DuckDBEngineAdapter)
+    adapter.set_current_catalog("test_catalog")
+
+    assert to_sql_calls(adapter) == [
+        'USE "test_catalog"',
+    ]
