@@ -16,6 +16,7 @@ export type ButtonVariant = Subset<
   | typeof EnumVariant.Warning
   | typeof EnumVariant.Alternative
   | typeof EnumVariant.Neutral
+  | typeof EnumVariant.Info
 >
 
 export type ButtonSize = Subset<
@@ -62,7 +63,7 @@ const VARIANT = new Map<ButtonVariant, string>([
   ],
   [
     'alternative',
-    'border-primary-500 bg-primary-10 hover:bg-primary-20 active:bg-primary-200 text-primary-500',
+    'border-neutral-300 bg-neutral-5 hover:bg-neutral-20 active:bg-neutral-200 text-neutral-600',
   ],
   [
     'secondary',
@@ -84,6 +85,10 @@ const VARIANT = new Map<ButtonVariant, string>([
     'neutral',
     'border-neutral-200 bg-neutral-200 hover:bg-neutral-300 active:bg-neutral-300 text-primary-900',
   ],
+  [
+    'info',
+    'border-transparent bg-neutral-10 dark:bg-neutral-20 active:bg-neutral-10 text-neutral-700 dark:text-neutral-200',
+  ],
 ])
 
 const SHAPE = new Map<ButtonShape, string>([
@@ -93,7 +98,7 @@ const SHAPE = new Map<ButtonShape, string>([
 ])
 
 const SIZE = new Map<ButtonSize, string>([
-  [EnumSize.xs, `px-2 py-0 text-xs leading-2 border`],
+  [EnumSize.xs, `text-xs leading-2 border`],
   [EnumSize.sm, `px-2 py-[0.125rem] text-xs leading-4 border-2`],
   [EnumSize.md, `px-3 py-2 text-base leading-6 border-2`],
   [EnumSize.lg, `px-4 py-3 text-lg border-4`],
@@ -103,7 +108,11 @@ const Button = makeButton(
   React.forwardRef<HTMLButtonElement, PropsButton>(ButtonPlain),
 )
 
-export { VARIANT, SHAPE, SIZE, Button, makeButton }
+const ButtonLink = makeButton(
+  React.forwardRef<HTMLDivElement, PropsButton>(ButtonLinkPlain),
+)
+
+export { VARIANT, SHAPE, SIZE, Button, ButtonLink, makeButton }
 
 function ButtonPlain(
   {
@@ -127,6 +136,14 @@ function ButtonPlain(
       form={form}
       disabled={disabled}
       onClick={onClick}
+      onKeyDown={(e: React.KeyboardEvent<HTMLButtonElement>) => {
+        if (e.key === 'Enter' || e.key === ' ') {
+          e.preventDefault()
+          e.stopPropagation()
+
+          onClick?.(e as unknown as React.MouseEvent<HTMLButtonElement>)
+        }
+      }}
       className={className}
     >
       {children}
@@ -134,6 +151,22 @@ function ButtonPlain(
   )
 }
 
+function ButtonLinkPlain(
+  { children = [], autoFocus, tabIndex, className }: PropsButton,
+  ref?: React.ForwardedRef<HTMLDivElement>,
+): JSX.Element {
+  return (
+    <div
+      ref={ref}
+      autoFocus={autoFocus}
+      tabIndex={tabIndex}
+      className={className}
+    >
+      {children}
+    </div>
+  )
+}
+
 function makeButton<TElement = HTMLButtonElement>(
   Component: React.ElementType,
 ): React.ForwardRefExoticComponent<
@@ -171,7 +204,7 @@ function makeButton<TElement = HTMLButtonElement>(
           SHAPE.get(shape),
           SIZE.get(size),
           disabled
-            ? 'opacity-50 bg-neutral-10 border-neutral-300 text-prose cursor-not-allowed'
+            ? 'opacity-50 bg-neutral-10 dark:bg-neutral-20 border-transparent text-prose cursor-not-allowed'
             : VARIANT.get(variant),
           className,
         )}
