@@ -30,12 +30,30 @@ export default defineConfig({
     globals: true,
     environment: 'jsdom',
     setupFiles: ['./src/tests/setup.ts'],
+    exclude: ['**/node_modules/**', './tests'],
   },
-  server: {
-    proxy: {
-      '/api': 'http://api:8000',
-    },
-  },
+  server:
+    process.env.NODE_ENV === 'testing'
+      ? {}
+      : {
+          proxy: {
+            '/api': {
+              target: 'http://api:8000',
+            },
+            '/docs': {
+              target: 'http://app:8001',
+              rewrite: path => '/',
+            },
+            '/data': {
+              target: 'http://app:8001',
+              rewrite: path => '/',
+            },
+            '/lineage': {
+              target: 'http://app:8001',
+              rewrite: path => '/',
+            },
+          },
+        },
   preview: {
     port: 8005,
   },
